package com.ascendfinalproject.warehouse.kafkaservice;

import com.ascendfinalproject.warehouse.models.Address;
import com.ascendfinalproject.warehouse.models.Item;
import com.ascendfinalproject.warehouse.models.WarehouseOrderRequest;
import com.ascendfinalproject.warehouse.models.WarehouseOrderResponse;
import com.ascendfinalproject.warehouse.services.WarehouseOrderService;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.event.ApplicationStartedEvent;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Mono;
import reactor.kafka.receiver.KafkaReceiver;

import java.util.Arrays;
import java.util.List;


@Service
@Slf4j
public class Receiver {

  @Autowired
  private KafkaReceiver<String, String> kafkaReceiver;

  @Autowired
  private WarehouseOrderService orderService;


  @EventListener(ApplicationStartedEvent.class)
  public void consumeNewOrder() {

    kafkaReceiver.receive()
            .doOnNext(record -> {
              try {
                System.out.println(record.value());
                ObjectMapper objectMapper = new ObjectMapper();
                WarehouseOrderRequest order = objectMapper.readValue(record.value(), WarehouseOrderRequest.class);
                log.info("ORDER objectMapper {}", order);
                Mono<WarehouseOrderResponse> mono = orderService.createOrder(order);
                mono.subscribe();
//                mono.block();
              } catch (Exception e) {
                log.error("Caught error", e);
              }
            })
            .doOnError(throwable -> System.out.println(throwable.getMessage()))
            .subscribe();
  }

}
