package com.ascendfinalproject.warehouse.services;

import com.ascendfinalproject.warehouse.exceptions.NotFoundException;
import com.ascendfinalproject.warehouse.models.Address;
import com.ascendfinalproject.warehouse.models.WarehouseOrderRequest;
import com.ascendfinalproject.warehouse.models.WarehouseOrderResponse;
import com.ascendfinalproject.warehouse.repositories.WarehouseOrderRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import reactor.core.CoreSubscriber;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.Date;

@Service
public class WarehouseOrderService {

  @Autowired
  WarehouseOrderRepository orderRepository;

  public Mono<WarehouseOrderResponse> getById(String id) {
    return orderRepository.findById(id);
  }

  public Flux<WarehouseOrderResponse> getOrders() {
    return orderRepository.findAll();
  }

  public Mono<WarehouseOrderResponse> createOrder(WarehouseOrderRequest order) {
    System.out.println("coming in from receiver " + order);

      WarehouseOrderResponse response = new WarehouseOrderResponse();


    response.setOrderId(order.getId());
    response.setStatus("RECEIVED");
//      response.setCreatedAt(order.getOrderCreatedAt());
//      response.setModifiedAt(System.currentTimeMillis());
    response.setCreatedAt(new Date(order.getOrderCreatedAt()));
    response.setModifiedAt(new Date(System.currentTimeMillis()));
    response.setOrderItems(order.getOrderItems());
    Address address = order.getCustomerAddress();
    response.setAddress(address.getStreet() + " " +
            address.getCity() + " " +
            address.getState() + " " +
            address.getZip()
    );

    System.out.println("built response " + response);
    System.out.println("-----------");

    return orderRepository.save(response);
  }


//  WarehouseOrderResponse{id='null', orderId='6085cb1e7681124ea05d2cab', status='RECEIVED', createdAt=Fri May 07 16:23:18 PDT 2021,
//          modifiedAt=Fri May 07 16:21:39 PDT 2021, orderItems=[com.ascendfinalproject.warehouse.models.Item@2bfa9ace,
//          com.ascendfinalproject.warehouse.models.Item@4a236e68, com.ascendfinalproject.warehouse.models.Item@8c53158,
//          com.ascendfinalproject.warehouse.models.Item@b50f9d1], address='390 15th st., New York, NY, 10010'}
//
//
//  ++++++++WarehouseOrderResponse(id=null, orderId=6085cb1e7681124ea05d2cab, status=RECEIVED, createdAt=Fri May 07 19:23:18 EDT 2021,
//                                 modifiedAt=Wed Dec 31 19:00:00 EST 1969, orderItems=[Item(itemId=1, itemName=item name, itemQuantity=5,
//                                 itemPrice=5.99, itemSku=1234)], address=390 15th st., New York, NY, 10010)


  public Mono<WarehouseOrderResponse> updateOrder(WarehouseOrderResponse order, String id) {
    return orderRepository.findById(id)
        .flatMap(existingOrder -> {
          existingOrder.setStatus(order.getStatus());
//          existingOrder.setModifiedAt(System.currentTimeMillis());
          existingOrder.setModifiedAt(new Date(System.currentTimeMillis()));
          return orderRepository.save(existingOrder);
        });
  }

  public Mono<Void> deleteOrder(String id) {
    return orderRepository.deleteById(id);
  }

  public Mono<Void> deleteAllOrders() {
    return orderRepository.deleteAll();
  }

}
